@@ -27,50 +27,6 @@ extern int delete_mode;
 
 static struct exclude_struct **exclude_list;
 
-/*
- * Optimization for special case when all included files are explicitly
- *   listed without wildcards in the "exclude" list followed by a "- *"
- *   to exclude the rest.
- * Contributed by Dave Dykstra <dwd@bell-labs.com>
- */
-static int only_included_files = 1;
-static struct exclude_struct *exclude_the_rest;
-
-int send_included_file_names(int f,struct file_list *flist)
-{
-	struct exclude_struct *ex, **ex_list;
-	int n;
-	char *p;
-
-	if (!only_included_files || (exclude_the_rest == NULL) || delete_mode)
-		return 0;
-
-	if (verbose > 1) {
-		rprintf(FINFO,"(using include-only optimization) ");
-	}
-
-	/* set exclude_list to NULL temporarily so check_exclude */
-	/*   will always return true */
-	ex_list = exclude_list;
-	exclude_list = NULL;
-	for (n=0; (ex = ex_list[n]) != NULL; n++) {
-		if (ex == exclude_the_rest)
-			break;
-		p = ex->pattern;
-		while (*p == '/') {
-			/* skip the allowed beginning slashes */
-			p++;
-		}
-		/* silently skip files that don't exist to
-		   be more like non-optimized case */
-		if (access(p,0) == 0)
-			send_file_name(f,flist,p,0,0);
-	}
-	exclude_list = ex_list;
-	
-	return 1;
-}
-
 /* build an exclude structure given a exclude pattern */
 static struct exclude_struct *make_exclude(char *pattern, int include)
 {
@@ -95,15 +51,18 @@ static struct exclude_struct *make_exclude(char *pattern, int include)
 	if (!ret->pattern) out_of_memory("make_exclude");
 
 	if (strpbrk(pattern, "*[?")) {
-	    if (!ret->include && (*pattern == '*') && (*(pattern+1) == '\0')) {
-		    exclude_the_rest = ret;
-	    } else {
-		    only_included_files = 0;
-	    }
 	    ret->regular_exp = 1;
-	    ret->fnmatch_flags = strstr(pattern, "**") ? 0 : FNM_PATHNAME;
-	} else if (!ret->include) {
-		only_included_files = 0;
+	    ret->fnmatch_flags = FNM_PATHNAME;
+	    if (strstr(pattern, "**")) {
+		    static int tested;
+		    if (!tested) {
+			    tested = 1;
+			    if (fnmatch("a/b/*", "a/b/c/d", FNM_PATHNAME)==0) {
+				    rprintf(FERROR,"WARNING: fnmatch FNM_PATHNAME is broken on your system\n");
+			    }
+		    }
+		    ret->fnmatch_flags = 0;
+	    }
 	}
 
 	if (strlen(pattern) > 1 && pattern[strlen(pattern)-1] == '/') {
@@ -145,15 +104,17 @@ static int check_one_exclude(char *name,struct exclude_struct *ex,
 	}
 
 	if (ex->regular_exp) {
-		if (fnmatch(pattern, name, ex->fnmatch_flags) == 0)
+		if (fnmatch(pattern, name, ex->fnmatch_flags) == 0) {
 			return 1;
+		}
 	} else {
 		int l1 = strlen(name);
 		int l2 = strlen(pattern);
 		if (l2 <= l1 && 
 		    strcmp(name+(l1-l2),pattern) == 0 &&
-		    (l1==l2 || (!match_start && name[l1-(l2+1)] == '/')))
+		    (l1==l2 || (!match_start && name[l1-(l2+1)] == '/'))) {
 			return 1;
+		}
 	}
 
 	return 0;
@@ -199,8 +160,6 @@ void add_exclude_list(char *pattern,struct exclude_struct ***list, int include)
 		}
 		free((*list));
 		*list = NULL;
-		only_included_files = 1;
-		exclude_the_rest = NULL;
 		return;
 	}
 
@@ -298,8 +257,7 @@ void send_exclude_list(int f)
 void recv_exclude_list(int f)
 {
 	char line[MAXPATHLEN];
-	unsigned int l;
-
+	int l;
 	while ((l=read_int(f))) {
 		if (l >= MAXPATHLEN) overflow("recv_exclude_list");
 		read_sbuf(f,line,l);
